#! /usr/users/goldnbrg/bin/perl -w

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# perl script to read all of the files in a directory
# with a specified suffix and treat these as 
# input files for Tim Richmond's ACCESS program
# Then uses summAcc.pl script to calculate summaries
# of different categories of accessible surface areas
# writes table to standard output 

# written by David P. Goldenberg 
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

# requires two input variables, suffix for input files
# and name for output file
# also accepts one option: -nfn
# when set eliminates output of file name on each line

# writes header, but only if the file does not already exist

# also sorts files in a "sensible way", that is
# by using numbers extracted from segments (up to 5)
# separated by dots.
# uses subroutines in dpg.pl for sorting

# Requires the following
#    perl library dpg.lib
#    The ACCESS binary
#    A parameter file for ACCESS access.par
#    The perl script summAcc.pl
# all of these are assumed to be in the same directory, $binDir

# Modify following line to point to correct directory
$binDir = '/usr/users/goldnbrg/goldnberg2/dyanaCalcs/dyanaExampleBpti/bin/';



$library = $binDir."dpg.pl";
require $library;
$ACCESS = $binDir."ACCESS";
$accessPar = $binDir."access.par";
$summAcc = $binDir."summAcc.pl";

$noFileName = 0;
if ($ARGV[0] eq '-nfn') {
    $noFileName = 1;
    shift(@ARGV);
}
$inSuf = $ARGV[0];
$outFile = $ARGV[1];



opendir THISDIR, ".";
@allfiles = readdir THISDIR;
closedir THISDIR;

@inFiles = grep(/\.$inSuf\Z/, @allfiles);

@inFiles = fileSort(@inFiles);

$maxNameLngth = 0;
foreach $name (@inFiles) {
  if (length($name) > $maxNameLngth) {
    $maxNameLngth = length($name);
  }
}

$nameFieldLngth = $maxNameLngth + 3;

if (-e $outFile) {
    open(OUTFILE, ">>" . $outFile);
} else {
    open(OUTFILE, ">" . $outFile);
    if (!$noFileName) {
	printf OUTFILE "%-${nameFieldLngth}s\t", "File";
    }
    print OUTFILE "Total\tNonPol\tPolar\tBckBn\tSdChn\tNPBB\tPBB\tNPSC\tPSC \n";

}



foreach $file (@inFiles) {

  `ln -sf $file fort.10`;

  `$ACCESS < $accessPar`;
  $outLine = `$summAcc -nh fort.11`;
  if (!$noFileName) {
      printf OUTFILE "%-${nameFieldLngth}s", $file;
  }
  print OUTFILE $outLine;
  `rm fort.11`;

}



