#! /usr/users/goldnbrg/bin/perl

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# A perl script to calculate backbone dihedral angles from dg format coordinate files
# written by Zhen Qin and David P. Goldenberg - Fall 2000


#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

# require 3 inputs:
# name of coordinate file
# name of file with a list of residue numbers
# name of output file
# If the output file does not already exist, the first output line is a header identifying
# the residues.  If this file exists, only data lines are output.
# Data are output as tab delimited dihedral angles in degrees


use POSIX;

if (@ARGV !=3) {
   die "Incorrecr number of input arguments. Input:coordinate file name, residue number file name,output file name\n";
}

$coorFile = shift(@ARGV);
$resiFile = shift(@ARGV);
$outFile = shift(@ARGV);

@residue = ();

open (RESIFILE, $resiFile);
$noResi = &readRESI(RESIFILE);
close (RESIFILE);


# print "\n\nThe Residues to be calculated are: @residue \n";
# print "The total number is:$noResi \n";

open (COORFILE,$coorFile);
$lastResi = &findlast(COORFILE);
close (COORFILE);
# print "Totol number of residue in the protein is: $lastResi\n";

open (COORFILE,$coorFile);
$noStruct = &scanCoor(COORFILE);
close (COORFILE);

# print "Total structure number: $noStruct \n";

if (-e $outFile){
  open(OUTFILE,">>".$outFile);
  &writeDihedrals(OUTFILE);
  close(OUTFILE);
} else {
  open(OUTFILE,">" . $outFile);
  &writeHeader(OUTFILE);
  &writeDihedrals(OUTFILE);
}


sub findlast{

  #this is a subroutine to find out the last residue from an already opened file

  my ($filehandle, $line, @field);
  $filehandle = $_[0];
  $last = 0;
  while ($line = <$filehandle>){
    @field = split(' ',$line);
  }
  $field[2];
}


sub readRESI{

  # this is a subroutine to read residue numbers from an already opened file
  # accepts one input variable, a string with the file handle
  # assumes each line containing only the number of the residue
  # lines beginning with "#" are ignored
  # returns number of the residues, determined as the number of elements in @residue

  my ($filehandle, $line, $noRESI, @field);
  $filehandle = $_[0];
  @residue = ();
  while($line = <$filehandle>) { 
    if(!($line =~ /^#/)) {
       @field=split(' ',$line);
       @residue = (@residue,$field[0]);
    }
  }
  $noRESI = @residue;
}
  

sub scanCoor{
 
  # this is a sunroutine that reads opened dg format corrdinate file
  # and scans for residues defined in global variable arrays @residue
  # the subroutine looks for multiple corrdinate sets in the file.
  # for each residue, the dihedral "phi" and "psi" are  calculated for each structure in the corrdinate file
  # the two dihedrals are stored in the global three dimensional array @dihedralPHI dihedralPSI

  my ($filehandle, @field, $line,$nofields);
  my (@atom0found, @atom1found, @atom2found, @atom3found, @atom4found);
  my ($iresidue, @count, @coor0,@coor1,@coor2,@coor3,@coor4);
  
  $filehandle = $_[0];
  seek($filehandle, 0,1);
  for( $iresidue=0;$iresidue<$noResi;$iresidue++) {
     $atom0found[$iresidue] = 0;   
     $atom1found[$iresidue] = 0;   
     $atom2found[$iresidue] = 0;   
     $atom3found[$iresidue] = 0;   
     $atom4found[$iresidue] = 0;   
     $count[$iresidue] = 0;
   } 
   @dihedralPHI = ();
   @dihedralPSI = ();


   while ($line = <$filehandle>){
     @field = split (' ',$line);
     $nofield = @field;
     if ($nofield==7) {  
       for($iresidue = 0; $iresidue < $noResi; $iresidue++) {
	 if(($field[2]==$residue[$iresidue]-1) && ($field[1] eq "C")) {
            $x[$iresidue][0] = $field[4];
	    $y[$iresidue][0] = $field[5];
	    $z[$iresidue][0] = $field[6];
            $atom0found[$iresidue] = 1;
	  } elsif (($field[2]== $residue[$iresidue]) && ($field[1] eq "N")) {
            $x[$iresidue][1] = $field[4];
            $y[$iresidue][1] = $field[5];
            $z[$iresidue][1] = $field[6];
	    $atom1found[$iresidue] = 1; 
	  } elsif (($field[2]== $residue[$iresidue]) && ($field[1] eq "CA")) {
            $x[$iresidue][2] = $field[4];
            $y[$iresidue][2] = $field[5];
            $z[$iresidue][2] = $field[6]; 
	    $atom2found[$iresidue] = 1;
	  } elsif (($field[2]== $residue[$iresidue]) && ($field[1] eq "C")) {
            $x[$iresidue][3] = $field[4];
            $y[$iresidue][3] = $field[5];
            $z[$iresidue][3] = $field[6]; 
	    $atom3found[$iresidue] = 1; 
	  } elsif (($field[2]==($residue[$iresidue]+1)) && ($field[1] eq "N")) {
            $x[$iresidue][4] = $field[4];
            $y[$iresidue][4] = $field[5]; 
            $z[$iresidue][4] = $field[6];
	    $atom4found[$iresidue] = 1;
	  }

	 if($residue[$iresidue] == 1){
#	   $dihedralPHI[$iresidue][$count[$iresidue]] = "";
	 }elsif ($atom0found[$iresidue] && $atom1found[$iresidue] && $atom2found[$iresidue] 
	     && $atom3found[$iresidue])  {
          $dihedralPHI[$iresidue][$count[$iresidue]]= &CalDihedral($x[$iresidue][0],$y[$iresidue][0],$z[$iresidue][0],
                                                                   $x[$iresidue][1],$y[$iresidue][1],$z[$iresidue][1],
                                                                   $x[$iresidue][2],$y[$iresidue][2],$z[$iresidue][2],
                                                                   $x[$iresidue][3],$y[$iresidue][3],$z[$iresidue][3]);
	}
        
         if($residue[$iresidue]==$lastResi){
#	   $dihedralPSI[$iresidue][$count[$iresidue]] = "";
	 }elsif ($atom1found[$iresidue] && $atom2found[$iresidue] && $atom3found[$iresidue]
             && $atom4found[$iresidue])  {
          $dihedralPSI[$iresidue][$count[$iresidue]]= &CalDihedral($x[$iresidue][1],$y[$iresidue][1],$z[$iresidue][1],
                                                                   $x[$iresidue][2],$y[$iresidue][2],$z[$iresidue][2],
                                                                   $x[$iresidue][3],$y[$iresidue][3],$z[$iresidue][3],                                                                    $x[$iresidue][4],$y[$iresidue][4],$z[$iresidue][4]);
	}
        
        if (($atom1found[$iresidue]&&$atom2found[$iresidue]&&$atom3found[$iresidue])&&($atom4found[$iresidue]||$residue[$iresidue]==$lastResi)) {
          $atom0found[$iresidue] = 0;  
          $atom1found[$iresidue] = 0;  
          $atom2found[$iresidue] = 0;  
          $atom3found[$iresidue] = 0;  
          $atom4found[$iresidue] = 0;         	
          $count[$iresidue]++;
	}
       }
     }
   }
   $noStruct = $count[0];         	
}


sub CalDihedral{
                                         # Calculate the dihedral using the 3-coordinate of the 4 atoms

  @point_i = ($_[0],$_[1],$_[2]);
  @point_j = ($_[3],$_[4],$_[5]);
  @point_k = ($_[6],$_[7],$_[8]);
  @point_n = ($_[9],$_[10],$_[11]);      # readin the cordinate of the points

  @r_ij = &vector(@point_i , @point_j);
  @r_jk = &vector(@point_j , @point_k);
  @r_kn = &vector(@point_k , @point_n);  # change to vectors

  @plane_ijk = &x_product(@r_ij , @r_jk); # the perpendicular line's vector of
  @plane_jkn = &x_product(@r_jk , @r_kn); # the two planes

  $dotpro_ijk_jkn = &dot_product(@plane_ijk , @plane_jkn);

  $length_ijk = sqrt(&dot_product(@plane_ijk,@plane_ijk));# calculate |rij X rjk|
  $length_jkn = sqrt(&dot_product(@plane_jkn,@plane_jkn));# calculate |rjk X rkn|

  $angle = acos($dotpro_ijk_jkn/($length_ijk*$length_jkn)); # calculate the angle in rad

  @xpro_ijk_jkn = &x_product(@plane_ijk , @plane_jkn);

  $xpro_dot_jk = &dot_product(@xpro_ijk_jkn , @r_jk);

  if($xpro_dot_jk < 0){
    $angle = 0-$angle;
  }                                     # judge the direction of the dihedral angle
  $angle = $angle*180/3.141592653;
}
 

sub vector{
  $x = $_[3]-$_[0];
  $y = $_[4]-$_[1];
  $z = $_[5]-$_[2];                      # calculate the vector between 2 points
  ($x,$y,$z);
}

sub dot_product{
  $x = $_[0]*$_[3]+$_[1]*$_[4]+$_[2]*$_[5]; # calculate the dot product of 
  $x;                                       # the vectors  
}  

sub x_product{
  $x = $_[1]*$_[5]-$_[2]*$_[4];
  $y = $_[2]*$_[3]-$_[5]*$_[0];
  $z = $_[0]*$_[4]-$_[3]*$_[1]; # calculate the cross product of the vectors 
  ($x,$y,$z);
}    

sub writeDihedrals{
  my ($iresidue, $struct, $filehandle);
  $filehandle = $_[0];
  for ($struct = 0; $struct < $noStruct; $struct++) {
      for ($iresidue = 0; $iresidue < ($noResi-1); $iresidue++){
        printf $filehandle "%8.1f\t",$dihedralPHI[$iresidue][$struct];
        printf $filehandle "%8.1f\t",$dihedralPSI[$iresidue][$struct];
      }
      printf $filehandle "%8.1f\t",$dihedralPHI[$noResi-1][$struct];
      printf $filehandle "%8.1f",$dihedralPSI[$noResi-1][$struct];
     print $filehandle "\n";
  }
}

sub writeHeader{                # write a head for the output file in case of a new established one
  my ($iresidue, $filehandle);
  $filehandle = $_[0];
  for ($iresidue = 0; $iresidue < ($noResi-1); $iresidue++){   
      printf $filehandle "%4d",$residue[$iresidue];
      print $filehandle " PHI \t";
      printf $filehandle "%4d",$residue[$iresidue];
      print $filehandle " PSI \t";    
    }
  printf $filehandle "%4d",$residue[$noResi-1];
  print $filehandle " PHI \t";
  printf $filehandle "%4d",$residue[$noResi-1];
  print $filehandle " PSI";

  print $filehandle "\n";
}
