#! /usr/users/goldnbrg/bin/perl

# A perl script to calculate radii of gyration for
# structures in a dg coordinate file.
# Uses only alpha carbon atoms
# Written by
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!



# reads file from standard input,
# writes to standard output.

# The coordinate file can contain multiple structures,
# and the program will try to figure out where each structure begins and ends
# But, it isn't very smart and can probably be confused by a badly formed file.

&scanCoor(STDIN);

foreach $value (@rgSq) {
    printf "%-6.3f \n", sqrt($value);
}


sub scanCoor {
    #subroutine reads opened dg format coordinate file
    #scans for CA atoms to calculate radius of gyration
    #The subroutine looks for multiple coordinate sets in the file,
    #but it isn't very smart. 
    #Keeps track of structures by residue number for first CA atom.
    #
    #Accepts one input variable, a string representing the file handle
    #returns array with squares of radii of gyration.

    my $filehandle, @field, $line, $noFields, @sum, $firstRes;
    my @X, @Y, @Z, $i;
    $filehandle = $_[0];
    
    while ($line = <$filehandle>) {
	@field = split(' ',$line);
	$noFields = @field;
	if ($noFields == 7 && $field[1] eq "CA") {
	    if ($firstRes == 0) {
		$structCnt = 1;
		$firstRes = $field[2];
		$resCnt = 1;
		@X = ();
		@Y = ();
		@Z = ();
		@sum = ();
		$X[0] = $field[4];
		$Y[0] = $field[5];
		$Z[0] = $field[6];
		$sum[0] = 0;
	    } elsif ($field[2] == $firstRes){
		$rgSq[$structCnt-1] = &calcSum(@sum)/($resCnt*$resCnt);
		$structCnt ++;
		$resCnt = 1;
		@X = ();
		@Y = ();
		@Z = ();
		@sum = ();
		$X[0] = $field[4];
		$Y[0] = $field[5];
		$Z[0] = $field[6];
		$sum[0] = 0;
	    } else {
		$resCnt ++;
		for ($i = 0; $i < @sum; $i++) {
		    $sum[$i] += &distSq 
			($X[$i], $Y[$i], $Z[$i],
			 $field[4], $field[5], $field[6]);
		}
		push(@X, $field[4]);
		push(@Y, $field[5]);
		push(@Z, $field[6]);
		push(@sum, 0);
	    }
	}
    }
    $rgSq[$structCnt-1] = &calcSum(@sum)/($resCnt*$resCnt);
}


sub distSq {
    $sumSq;
    $sumSq = ($_[0] - $_[3]) * ($_[0] - $_[3]);
    $sumSq +=  ($_[1] - $_[4]) * ($_[1] - $_[4]);
    $sumSq +=  ($_[2] - $_[5]) * ($_[2] - $_[5]);
    return $sumSq;
}

sub calcSum {
    my $sum,$value;
    $sum = 0;
    foreach $value (@_) {
	$sum += $value;
    }
    return $sum;
}
