#! /usr/users/goldnbrg/bin/perl
# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# A perl script to read tabulated values
# calculate mean, harmonic mean and mean squared values and generate
# binned data to make a histogram
#
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!





# modified 12 August 03 to also calculate harmonic mean
# harmonic mean = N/(x1 + x2 + x3 .  . . .)
# harmonic mean calculation ignores 0 values.


# assumes that the input file is tab delimited
# the first line is interpretted as labels for the columnar data
# all subsequent lines are assumed to be individual data values

# The input is from stdin

# optionally, the number of bins to be created may be specified with 
# "-b value"
# and the maximamum value may be specified with
# "-m value"
# If not specified, 100 bins are created and the maximum value
# is taken to be 100

# Additional option, -c, calculates cumulative count and cumulative percent of
# total

# Results are written to stdout


$binCount = 100;
$maxVal = 100;
$cumMode = 0;

while (@ARGV) {
    $nextArg = shift(@ARGV);
    if ($nextArg eq "-b") {
	$binCount = shift(@ARGV);
    } elsif ($nextArg eq "-m") {
	$maxVal = shift(@ARGV);
    } elsif ($nextArg eq "-c") {
	$cumMode = 1;
    }
}

$interval = $maxVal/$binCount;



#read first line of data file for column names

$headerLine = <STDIN>;
chop($headerLine);
@names = split(/\t/, $headerLine);

$colNo = @names ;

@bin = ();

for ($iCol = 0; $iCol < $colNo; $iCol++) {
    for ($iBin = 0; $iBin < $binCount; $iBin++){
	$bin[$iCol][$iBin] = 0.0;
    }
    $sum[$iCol] = 0;
    $sumSq[$iCol] = 0;
    $dataCount[$iCol] = 0;
    $dataHarmMeanCt[$iCol] = 0;
    $sumRecip[$iCol] = 0;
}

# Read first data line of input file to initialize min and max 
$dataLine = <STDIN>;
@datum = split(/\t/, $dataLine);
for ($iCol = 0; $iCol < $colNo; $iCol++) {
    if ($datum[$iCol] =~ /\d/) {
    $binNo = int($datum[$iCol]/$interval);
    $bin[$iCol][$binNo]++;
    $sum[$iCol] += $datum[$iCol];
    $sumSq[$iCol] += $datum[$iCol]**2;
    if ($datum[$iCol] != 0) {
		$sumRecip[$iCol] += 1/$datum[$iCol];
		$dataHarmMeanCt[$iCol]++;
    }
    $dataCount[$iCol]++;
    $min[$iCol] = $datum[$iCol];
    $max[$iCol] = $datum[$iCol];
    }
}


while ($dataLine = <STDIN>) {
    @datum = split(/\t/, $dataLine);
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	if ($datum[$iCol] =~ /\d/) {
	    $binNo = int($datum[$iCol]/$interval);
	    $bin[$iCol][$binNo]++;
	    $sum[$iCol] += $datum[$iCol];
	    $sumSq[$iCol] += $datum[$iCol]**2;
	    if ($datum[$iCol] != 0) {
		$sumRecip[$iCol] += 1/$datum[$iCol];
		$dataHarmMeanCt[$iCol]++;
	    }
	    $dataCount[$iCol]++;
	    if ($datum[$iCol] < $min[$iCol]) {
		$min[$iCol] = $datum[$iCol]
	    }	    
	    if ($datum[$iCol] > $max[$iCol]) {
		$max[$iCol] = $datum[$iCol]
	    }

	}
    }
}


for ($iCol = 0; $iCol < $colNo; $iCol++) {
    $mean[$iCol] = $sum[$iCol]/$dataCount[$iCol];
    $meanSq[$iCol] = $sumSq[$iCol]/$dataCount[$iCol];
    if ($sumRecip[$iCol] != 0) {
	$harmonMean[$iCol] = $dataHarmMeanCt[$iCol]/$sumRecip[$iCol];
    } else {
	$harmonMean[$iCol] = 0;
    }
}


printf "%12s\t%-11s\t%-11s\t%-11s\t%-11s\t%-11s\t%-11s\n", "", "Mean", "Mean Sq", "HarmonMean", "Minimum", "Maximum", "Count";
for ($iCol = 0; $iCol < $colNo; $iCol++) {
    printf "#%11s\t", $names[$iCol];
    printf "%6.4g%6s\t", $mean[$iCol];    
    printf "%6.4g%6s\t", $meanSq[$iCol];
    printf "%6.4g%6s\t", $harmonMean[$iCol];
    printf "%6.4g%6s\t", $min[$iCol];    
    printf "%6.4g%6s\t", $max[$iCol];
    printf "%-5i%6s\t", $dataCount[$iCol];
    print "\n";
}


print "Int. max \t";
foreach $name (@names){
    print "$name \t"; 
    if ($cumMode) {
	print "cumm. \t";
	print "cumm % \t";
    }

}

print "\n";

@cumm = ();

for ($iBin = 0; $iBin < $binCount; $iBin++) {
    printf "%5.3f\t", ($iBin + 1)*$interval;
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	printf "%-5i%6s\t", $bin[$iCol][$iBin], "";
	if ($cumMode) {
	    $cumm[$iCol] += $bin[$iCol][$iBin];
	    $cumPercent = 100*$cumm[$iCol]/$dataCount[$iCol];
	    printf  "%-5i\t %-5.1f\t", $cumm[$iCol], $cumPercent;
	}
    }
    print "\n";
}



