#! /usr/users/goldnbrg/bin/perl 

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# perl script to read output of Tim Richmond's ACCESS program
# and generate summary with atoms grouped in various ways

# written by David P. Goldenberg 
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

# accepts one required input variable, 
# name of ACCESS output file, and
# and two optional flags, -r to list values by residue
# and -nh to not print header line
# and writes to standard output

$byRes = 0;
$noHead = 0;
foreach $arg (@ARGV) {
  if ($arg eq '-r') {
    $byRes = 1;
  } elsif ($arg eq '-nh') {
    $noHead = 1;
  } else {
    $inFile = $arg;
  }
}

if (!$noHead) {
  if ($byRes) {
    print "Res. no.  ";
  }
  print "  Total \t NonPol \t  Polar \t  BckBn \t  SdChn \t   NPBB  \t   PBB  \t  NPSC  \t   PSC \n";
}

$total = 0;
$totalPol = 0;
$totalNp = 0;
$totalBb = 0;
$totalSc = 0;
$totalNpBb = 0;
$totalPolBb = 0;
$totalNpSc = 0;
$totalPolSc = 0;

open (INFILE, $inFile);
$resNo = 0;
while ($line = <INFILE>) {
  @inField = split(' ', $line);
  $noFields = 5;
  if ($noFields == 5 && $inField[0] !~ /\#/) {
    if ($resNo != $inField[1]) {
      #new residue
      if ($byRes && $resNo != 0) {
        &writeLine ($resType, $resNo, $resTotal,
                    $resNp, $resPol, $resBb, $resSc,
                    $resNpBb, $resPolBb, $resNpSc, $resPolSc);
      }              
      $resNo = $inField[1];
      $resType = $inField[2];
      $resTotal = 0;
      $resPol = 0;
      $resNp = 0;
      $resBb = 0;
      $resSc = 0;
      $resNpBb = 0;
      $resPolBb = 0;
      $resNpSc = 0;
      $resPolSc = 0;
    }
    $asa = $inField[4];
    $atomType = $inField[3];
    # classify atoms
    if ($atomType eq 'N' || $atomType eq 'CA' ||
        $atomType eq 'C' || $atomType eq 'O' ||
        $atomType eq 'OXT' || $atomType eq 'OA' ||
        $atomType eq 'OB') {
      $bbAtom = 1;
    } else {
      $bbAtom = 0;
    }
    if ($atomType =~ /^[CS]/) {
      $polAtom = 0;
    } else {
      $polAtom = 1;
    }
    # add stuff up
    $total += $asa;
    $resTotal += $asa;
    if ($polAtom) {
      $totalPol += $asa;
      $resPol += $asa;
    } else {
      $totalNp += $asa;
      $resNp += $asa;
    }
    if ($bbAtom) {
      $totalBb += $asa;
      $resBb += $asa;
      if ($polAtom) {
        $totalPolBb += $asa;
        $resPolBb += $asa;
      } else {
        $totalNpBb += $asa;
        $resNpBb += $asa;
      }
    } else {
      $totalSc += $asa;
      $resSc += $asa;
      if ($polAtom) {
        $totalPolSc += $asa;
        $resPolSc += $asa;
      } else {
        $totalNpSc += $asa;
        $resNpSc += $asa;
      }
    }
  }
}
if ($byRes && $resNo != 0) {
  &writeLine ($resType, $resNo, $resTotal,
              $resNp, $resPol, $resBb, $resSc,
              $resNpBb, $resPolBb, $resNpSc, $resPolSc);
  print "Total   ";
}              

&writeLine ($total,
            $totalNp, $totalPol, $totalBb, $totalSc,
            $totalNpBb, $totalPolBb, $totalNpSc, $totalPolSc);


sub writeLine {
  $fields = @_;
  if ($fields == 11) {
    printf "%-5s\t", shift (@_);
    printf "%-3d\t", shift (@_);
  }
  foreach $value (@_) {
    printf "%-7.2f\t", $value;
  }
  printf "\n";
}

