Copyright (c) 1996-97 ETH Zurich
c     ==================================================================
c     UTIL:     Utility routines.
c
c               seterr   set error flag
c               haverr   check error condition
c               havint   check for interrupt
c               handlr   interrupt handler
c               icoman   complete abbreviated names
c               squeze   remove multiple blanks
c               range    read range of numbers
c               appext   append default extension to a filename
c               isort    sort array of integer numbers
c               rsort    sort array of real numbers
c               ran1     random number generator
c               gasdev   normally distributed random numbers
c               lookr    real number lookup table
c               looks    character lookup table
c
c     ==================================================================
      function iif(cond,itrue,ifalse)
      logical cond
      if (cond) then
        iif=itrue
      else
        iif=ifalse
      end if
      end
c     ==================================================================
      subroutine seterr
      logical erflag,intrpt
      common /errdat/erflag,intrpt
      erflag=.true.
      end
c     ==================================================================
      logical function haverr()
      logical erflag,intrpt
      common /errdat/erflag,intrpt
      haverr=erflag
      end
c     ==================================================================
      logical function havint()
      logical erflag,intrpt
      common /errdat/erflag,intrpt
      havint=intrpt
      end
c     ==================================================================
      subroutine handlr (isignl)
      logical erflag,intrpt
      common /errdat/erflag,intrpt
      intrpt=.true.
      end
c     ==================================================================
      subroutine split (str,field,nfield,maxfld)
      character*(*) str,field(maxfld)
      nfield=0
      j=0
      l=lenstr(str)
      do i=1,l
        if (str(i:i).gt.' ') then
          if (i.eq.1) then
            j=1
          else if (str(i-1:i-1).le.' ') then
            j=i
          end if
          if (str(i+1:i+1).le.' ') then
            nfield=nfield+1
            field(nfield)=str(j:i)
            if (nfield.ge.maxfld) return
          end if
        end if
      end do
      end
c     ==================================================================
      function icoman(cmd,comand,ncmd)
      character*(*) cmd,comand(ncmd)
      lcmd=min(lenstr(cmd),len(comand(1)))
      icoman=0
      do 20 i=1,ncmd
        if (cmd.eq.comand(i)(1:lcmd)) then
          if (icoman.ne.0 .and. cmd.ne.comand(i)) then
            icoman=-1
            return
          end if
          icoman=i
          if (cmd.eq.comand(i)) return
        end if
   20   continue
      end
c     ==================================================================
      function inlist(val,list)
c
      character*(*) val,list,fmt*250,str*250
c
      lval=lenstr(val)
      lfmt=lenstr(list)
      mode=iif(list(lfmt:lfmt).eq.'*',1,0)
      if (mode.eq.1) lfmt=lfmt-1
      fmt=list(1:lfmt)
      str(1:lval+1)='|'//val(1:lval)
      if (val(1:lval).eq.fmt(1:lval)) then
        k=1
      else
        k=index(fmt(1:lfmt),str(1:lval+1))
        if (k.gt.0) k=k+1
      end if
      if (k.gt.0 .and.
     *    (fmt(k+lval:k+lval).gt.' ' .and.
     *     fmt(k+lval:k+lval).ne.'|' .and.
     *     index(fmt(k+1:lfmt),str(1:lval+1)).gt.0)) k=-1
      if (k.gt.0) then
        if (mode.ne.0) then
          l=index(fmt(k:lfmt),'|')
          if (l.le.0) then
            val=fmt(k:lfmt)
          else
            val=fmt(k:k+l-2)
          end if
        end if
        inlist=1
        do j=1,k
          if (fmt(j:j).eq.'|') inlist=inlist+1
        end do
      else
        inlist=k
      end if
      end
c     ==================================================================
      subroutine elemnt (i,list,sep,value)
c
      character quote
      parameter (quote='''')
      logical quoted
      character*(*) list,sep,value
c
      m=0
      k=0
      l=lenstr(list)
      quoted=.false.
      do j=1,l
        if (list(j:j).eq.quote) quoted=.not.quoted
        if (j.eq.l .or. (list(j:j).eq.sep .and. .not.quoted)) then
          m=m+1
          if (m.eq.i) then
            if (j.eq.l) then
              value=list(k+1:j)
            else
              value=list(k+1:j-1)
            end if
            return
          end if
          k=j
        end if
      end do
      value=' '
      end
c     ==================================================================
      subroutine squeze (s)
      character*(*) s
      l=0
      do 10 i=1,len(s)
        if (s(max(1,i-1):i).ne.' ') then
          l=l+1
          s(l:l)=s(i:i)
        end if
   10   continue
      if (l.lt.len(s)) s(l+1:)=' '
      end
c     ==================================================================
      subroutine range (str,ibeg,iend,illexp)
c
      logical illexp
      character*(*) str
c
      illexp=.false.
      if (str.eq.'*') return
      jbeg=ibeg
      jend=iend
      i=index(str,'..')
      if (i.eq.0) then
        ibeg=ieva(str,illexp)
        iend=ibeg
      else
        l=lenstr(str)
        illexp=l.le.2
        if (i.gt.1 .and. .not.illexp) ibeg=ieva(str(1:i-1),illexp)
        if (i.lt.l-1 .and. .not.illexp) iend=ieva(str(i+2:),illexp)
      end if
      illexp=illexp .or. ibeg.lt.jbeg .or. ibeg.gt.jend .or.
     *                   iend.lt.ibeg .or. iend.gt.jend
      end
c     ==================================================================
      subroutine ranger (str,rbeg,rend,illexp)
c
      logical illexp
      character*(*) str
c
      illexp=.false.
      if (str.eq.'*') return
      sbeg=rbeg
      send=rend
      i=index(str,'..')
      if (i.eq.0) then
        rbeg=reva(str,illexp)
        rend=rbeg
      else
        l=lenstr(str)
        illexp=l.le.2
        if (i.gt.1 .and. .not.illexp) rbeg=reva(str(1:i-1),illexp)
        if (i.lt.l-1 .and. .not.illexp) rend=reva(str(i+2:),illexp)
      end if
      illexp=illexp .or. rbeg.lt.sbeg .or. rbeg.gt.send .or.
     *                   rend.lt.rbeg .or. rend.gt.send
      end
c     ==================================================================
      subroutine appext (filnam,defext)
c
      character*(*) filnam,defext
c
      if (filnam.eq.' ') return
      l=lenstr(filnam)
      do 10 i=1,l
   10   if (filnam(i:i).gt.' ') go to 20
   20 do 25 j=i,l
   25   filnam(j-i+1:j-i+1)=filnam(j:j)
      l=l-i+1
      if (filnam(l:l).eq.'|') then
        filnam(l:l)=' '
        return
      end if
      if (filnam.eq.'*') return
      do 30 i=l,1,-1
   30   if (filnam(i:i).eq.defext(1:1)) go to 40
   40 do 50 j=l,1,-1
   50   if (index(':]/',filnam(j:j)).gt.0) go to 60
   60 if (i.le.j) filnam(l+1:)=defext
      end
c     ==================================================================
      subroutine isort (key,indx,n,mode)
c
      parameter (nstack=30)
      dimension key(n),indx(n),isl(nstack),isr(nstack)
c
      if (mode.eq.0) then
        do 10 i=1,n
   10     indx(i)=i
      end if
      js=1
      isl(1)=1
      isr(1)=n
   50 continue
        jl=isl(js)
        jr=isr(js)
        js=js-1
   52   continue
          i=jl
          j=jr
          ix=key((jl+jr)/2)
   53     continue
   54       if (key(i).lt.ix) then
              i=i+1
            go to 54
            end if
   55       if (key(j).gt.ix) then
              j=j-1
            go to 55
            end if
            if (i.gt.j) go to 58
            k=key(i)
            key(i)=key(j)
            key(j)=k
            k=indx(i)
            indx(i)=indx(j)
            indx(j)=k
            i=i+1
            j=j-1
          if (i.le.j) go to 53
   58     if (j-jl.lt.jr-i) then
            if (i.lt.jr) then
              js=js+1
              isl(js)=i
              isr(js)=jr
            end if
            jr=j
          else
            if (jl.lt.j) then
              js=js+1
              isl(js)=jl
              isr(js)=j
            end if
            jl=i
          end if
        if (jl.lt.jr) go to 52
      if (js.gt.0) go to 50
      end
c     ==================================================================
      subroutine rsort (akey,indx,n,mode)
c
      parameter (nstack=30)
      dimension akey(n),indx(n),isl(nstack),isr(nstack)
c
      if (mode.eq.0) then
        do 10 i=1,n
   10     indx(i)=i
      end if
      js=1
      isl(1)=1
      isr(1)=n
   50 continue
        jl=isl(js)
        jr=isr(js)
        js=js-1
   52   continue
          i=jl
          j=jr
          xx=akey((jl+jr)/2)
   53     continue
   54       if (akey(i).lt.xx) then
              i=i+1
            go to 54
            end if
   55       if (akey(j).gt.xx) then
              j=j-1
            go to 55
            end if
            if (i.gt.j) go to 58
            t=akey(i)
            akey(i)=akey(j)
            akey(j)=t
            k=indx(i)
            indx(i)=indx(j)
            indx(j)=k
            i=i+1
            j=j-1
          if (i.le.j) go to 53
   58     if (j-jl.lt.jr-i) then
            if (i.lt.jr) then
              js=js+1
              isl(js)=i
              isr(js)=jr
            end if
            jr=j
          else
            if (jl.lt.j) then
              js=js+1
              isl(js)=jl
              isr(js)=j
            end if
            jl=i
          end if
        if (jl.lt.jr) go to 52
      if (js.gt.0) go to 50
      end
c     ==================================================================


c     ==================================================================
      function ran1(idum)
c
c     modified random number generator for much longer periodicity
c     patch from Peter Guentert 22 July 1998

      parameter (im1=2147483563,im2=2147483399,am=1.0/im1,imm1=im1-1,
     *           ia1=40014,ia2=40692,iq1=53668,iq2=52774,ir1=12211,
     *           ir2=3791,ntab=32,ndiv=1+imm1/ntab,eps=1.2e-7,
     *           rnmx=1.0-eps)
      dimension iv(ntab)
      save iv,iy,idum2
c
      data idum2/123456789/, iv/ntab*0/, iy/0/
c
      if (idum.le.0) then
        idum=max(-idum,1)
        idum2=idum
        do j=ntab+8,1,-1
          k=idum/iq1
          idum=ia1*(idum-k*iq1)-k*ir1
          if (idum.lt.0) idum=idum+im1
          if (j.le.ntab) iv(j)=idum
        end do
        iy=iv(1)
      end if
      k=idum/iq1
      idum=ia1*(idum-k*iq1)-k*ir1
      if (idum.lt.0) idum=idum+im1
      k=idum2/iq2
      idum2=ia2*(idum2-k*iq2)-k*ir2
      if (idum2.lt.0) idum2=idum2+im2
      j=1+iy/ndiv
      iy=iv(j)-idum2
      iv(j)=idum
      if(iy.lt.1)iy=iy+imm1
      ran1=min(am*iy,rnmx)
      end

c     ==================================================================
      function gasdev(idum)
c
      data iset/0/
      save gset
c
      if (iset.eq.0 .or. idum.lt.0) then
    1   v1=2.0*ran1(idum)-1.0
        v2=2.0*ran1(idum)-1.0
        r=v1**2+v2**2
        if (r.ge.1.0) go to 1
        fac=sqrt(-2.0*log(r)/r)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      end if
      end
c     ==================================================================
      function lookr (r,tab,n,mode)
      dimension tab(n)
      logical ascend
      lookr=0
      if (n.lt.1) return
      ascend=tab(n).gt.tab(1)
      ju=n+1
   10 if (ju-lookr.gt.1) then
        m=(ju+lookr)/2
        if (ascend .eqv. r.ge.tab(m)) then
          lookr=m
        else
          ju=m
        end if
      go to 10
      end if
      if (mode.eq.0 .and. lookr.gt.0) then
        if (r.ne.tab(lookr)) lookr=0
      end if
      end
c     ==================================================================
      function looks (s,tab,n,mode)
      character*(*) s,tab(n)
      logical ascend
      looks=0
      if (n.lt.1) return
      ascend=tab(n).gt.tab(1)
      ju=n+1
   10 if (ju-looks.gt.1) then
        m=(ju+looks)/2
        if (ascend .eqv. s.ge.tab(m)) then
          looks=m
        else
          ju=m
        end if
      go to 10
      end if
      if (mode.eq.0 .and. looks.gt.0) then
        if (s.ne.tab(looks)) looks=0
      end if
      end
